package org.greenstone.gatherer.metadata;


import java.awt.*;
import java.io.*;
import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import org.greenstone.gatherer.Configuration;
import org.greenstone.gatherer.Gatherer;
import org.greenstone.gatherer.file.FileNode;


public class MetadataAuditTableModel
    extends AbstractTableModel
{
    /** The FileNodes this model is built for */
    private FileNode[] file_nodes = null;
    /** The list of MetadataElements in the table (columns) */
    private ArrayList metadata_elements = new ArrayList();
    /** The list of MetadataAuditTableEntries in the table (rows) */
    private ArrayList metadata_audit_table_entries = new ArrayList();


    /** Returns the number of columns in this table. */
    public int getColumnCount()
    {
	return metadata_elements.size();
    }


    /** Retrieves the name of the specified column. */
    public String getColumnName(int col)
    {
        return metadata_elements.get(col).toString();
    }


    /** Returns the number of rows in this table. */
    public int getRowCount()
    {
	return metadata_audit_table_entries.size();
    }


    /** Returns the cell value at a given row and column as an Object. */
    public Object getValueAt(int row, int col)
    {
	// Check values are reasonable
	if (row < 0 || row >= metadata_audit_table_entries.size() || col < 0 || col >= metadata_elements.size()) {
	    return null;
	}

	MetadataAuditTableEntry metadata_audit_table_entry = (MetadataAuditTableEntry) metadata_audit_table_entries.get(row);
	return metadata_audit_table_entry.getMetadataValueTreeNode(col);
    }


    public void rebuild(FileNode[] file_nodes)
    {
	metadata_elements = MetadataSetManager.getEveryMetadataSetElement();
	metadata_audit_table_entries.clear();

	this.file_nodes = file_nodes;
	if (file_nodes != null && file_nodes.length > 0) {
	    // Create model builder
	    MetadataAuditTableModelBuilder builder = new MetadataAuditTableModelBuilder();
	    builder.run();
	}
    }


    private class MetadataAuditTableModelBuilder
    {
	public void run()
	{
	    // Build a list of MetadataAuditTableEntries that represent the metadata asssigned to the selected files
	    System.err.println("Building MetadataAuditTableModel...");

	    // Process each of the selected files in turn
	    for (int i = 0; i < file_nodes.length; i++) {
		File current_file = file_nodes[i].getFile();
		System.err.println("  File: " + current_file);

		// Get the metadata assigned to this file
		ArrayList current_file_metadata = MetadataXMLFileManager.getMetadataAssignedToFile(current_file);

		// Get the extracted metadata for this file, if desired
		if (Gatherer.config.get("general.view_extracted_metadata", Configuration.COLLECTION_SPECIFIC) == true) {
		    current_file_metadata.addAll(DocXMLFileManager.getMetadataExtractedFromFile(current_file));
		}

		MetadataAuditTableEntry metadata_audit_table_entry = new MetadataAuditTableEntry(current_file, current_file_metadata);
		metadata_audit_table_entries.add(metadata_audit_table_entry);
	    }
	}
    }


    /** This class represents one (file, metadata) pair */
    private class MetadataAuditTableEntry
    {
	private File file = null;
	private MetadataValue[] metadata_values = null;


	public MetadataAuditTableEntry(File file, ArrayList metadata_values_list)
	{
	    this.file = file;

	    metadata_values = new MetadataValue[metadata_elements.size()];
	    for (int i = 0; i < metadata_values_list.size(); i++) {
		MetadataValue metadata_value = (MetadataValue) metadata_values_list.get(i);
		MetadataElement metadata_element = metadata_value.getMetadataElement();

		// Find the right column to put the value in
		for (int j = 0; j < metadata_elements.size(); j++) {
		    if (metadata_element.equals(metadata_elements.get(j))) {
			metadata_values[j] = metadata_value;
			break;
		    }
		}
	    }
	}


	public MetadataValueTreeNode getMetadataValueTreeNode(int col)
	{
	    if (metadata_values[col] == null) {
		return null;
	    }

	    return metadata_values[col].getMetadataValueTreeNode();
	}
    }
}
